<?php
/**
 * @file
 * Image compession config form.
 */
namespace Drupal\image_compression\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\social_media_links\Plugin\SocialMediaLinks\Platform\Drupal;

class ImageCompression extends ConfigFormBase {
  protected $number = 1;
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'form_compression';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'image_compression.settings',
    ];
  }
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('image_compression.settings');
    $form['#tree'] = TRUE;
    $form['compress'] = [
      '#title'      => 'Compression settings',
      '#type'       => 'fieldset',
      '#attributes' => ['id' => 'compressor'],
    ];
    for ($i = 1; $i <= $this->number; $i++) {
      $form['compress']['field_size_' . $i] = [
        '#type'       => 'textfield',
        '#title' => 'Enter the filesize of images above which to be considered',
        '#attributes' => ['placeholder' => $this->t('Enter the filesize')],
        '#default_value' => $config->get('field_size_' . $i),
        '#size'       => 20,
      ];
      $form['compress']['field_rate_'. $i] = [
        '#type' => 'select',
        '#options' => [
          '75' => $this
            ->t('75'),
          '70' => $this
            ->t('70'),
          '65' => $this
            ->t('65'),
          '60' => $this
            ->t('60'),
          '50' => $this
            ->t('50'),
          '40' => $this
            ->t('40'),
          '30' => $this
            ->t('30'),
          '25' => $this
            ->t('25'),
        ],
        '#title' => $this->t('Enter the compression rate.'),
        '#default_value' => $config->get('field_rate_' . $i),
      ];
    }

    $form['compress']['actions']['add_item'] = [
      '#type'   => 'submit',
      '#value'  => $this->t('Add another option'),
      '#submit' => ['::image_compression_add_item'],
      '#ajax'   => [
        'callback' => '::image_compression_ajax_callback',
        'wrapper'  => 'compressor',
      ],
    ];
    return parent::buildForm($form, $form_state);
  }
  /**
   * {@inheritdoc}
   */
  public function image_compression_ajax_callback($form, $form_state) {
    return $form['compress'];
  }

  /**
   * {@inheritdoc}
   */
  public function image_compression_add_item(array &$form, FormStateInterface $form_state) {
    $this->number++;
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    parent::submitForm($form, $form_state);
    $form_state_input = $form_state->getUserInput();
    for ($i = 1; $i <= $this->number; $i++) {
      $this->config('image_compression.settings')
        ->set('field_size_' . $i, $form_state_input['compress']['field_size_' . $i])
        ->set('field_rate_' . $i, $form_state_input['compress']['field_rate_' . $i])
        ->save();
    }
  }
}
