<?php

namespace Drupal\image_compression;

use Drupal\Core\Image\Image;
use Drupal\social_media_links\Plugin\SocialMediaLinks\Platform\Drupal;

class ImageCompressionServices
{
  /**
   * Function to get all the files inside a directory and subdirectory.
   *
   * @param string $dir
   *   Path to the files directory.
   * @param array $results
   *   Array which stores all the files inside a directory.
   *
   * @return array
   *   Returns the array of files.
   */
  public function get_dir_contents($dir)
  {
    $item = new \RecursiveDirectoryIterator($dir);
    foreach (new \RecursiveIteratorIterator($item) as $filename => $file) {
      $results[] = $filename;
    }
    foreach ($results as $key => $value) {
      if (filesize($value) == 0) {
        continue;
      }
      if ($value != "." && $value != "..") {
        $res[] = $value;
      }
    }
    $img_files = preg_grep('~\.(jpeg|jpg|png)$~', $res);
    return $img_files;
  }

  /**
   * Function which implements compress_images().
   */
  public function compress_images()
  {
    $dir = 'sites/default/files';
    $file = $this->get_dir_contents($dir);
    $batch = [
      'title' => t("Image Compression"),
      'operations' => [
        [
          '\Drupal\image_compression\ImageCompressionServices::imageCompressionBatchProcessing', [$file],
        ],
      ],
      'finished' => '\Drupal\image_compression\ImageCompressionServices::imageCompressionBatchProcessFinished',
    ];
    batch_set($batch);
  }

  /**
   * Function to compress png images.
   *
   * @param string $src
   *   The source path to the file.
   * @param int $quality
   *   The quality for the file.
   *
   * @return mixed
   *   Returns a compressed file.
   */
  public function compress_png($src, $quality) {
    $source = imagecreatefrompng($src);
    $destination = $src;
    imagepng($source, $destination, $quality);
    return $destination;
  }

  /**
   * Function to compress jpeg/jpg images.
   *
   * @param string $source
   *   The source path to the file.
   * @param int $quality
   *   The quality for the file.
   *
   * @return array
   *   Returns a compressed file.
   */
  public function compress_jpg($source, $quality) {
    $src = imagecreatefromjpeg($source);
    $dest = $source;
    imagejpeg($src, $dest, $quality);
    return $dest;
  }
  /**
   * Function to compress images.
   *
   * @param object $file
   *   Object to implement compress.
   *
   * @return mixed
   *   returns a file which is compressed.
   */
  public function compress($file)
  {
    $config = \Drupal::config('image_compression.settings');
    $i = 1;
    while(($config->get('field_size_'.$i)) != "") {
      $file_size[$i] = $config->get('field_size_'.$i);
      $quality[$i] = $config->get('field_rate_'. $i);
      $i++;
    }
    for ( $i = 1 ; $i<=count($file_size);$i++) {
      $serial_data[$file_size[$i]] = $quality[$i];
    }
      rsort($file_size);
    if ($file) {
      $source = $file->getFileUri();
      $size = $file->getSize();
      $info = getimagesize($source);
      switch ($info['mime']) {
        case 'image/jpg':
        case 'image/jpeg':
          for ( $i = 0 ; $i<count($file_size);$i++) {
            $current_value = $file_size[$i];
            if ($size > $current_value) {
              $this->compress_jpg($source, $serial_data[$current_value]);
            }
          }
          break;

        case 'image/png':
          $this->compress_png($source, 9);
          break;
      }
      return $source;
    }
  }

  /**
   * Function for batch process.
   *
   * @param array $files
   *   Contains all the file name.
   * @param array $context
   *   To store progress result.
   */
  public static function imageCompressionBatchProcessing(array $files, array &$context)
  {
    $message = "Image compressing...";
    if (empty($context['sandbox'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['current_id'] = 0;
      $context['sandbox']['max'] = count($files);
    }
    $limit = 10;
    $sliced_datas = array_slice($files, $context['sandbox']['current_id'], $limit);
    foreach ($sliced_datas as $value) {
      $context['sandbox']['progress']++;
      $filename = basename($value);
      $files = \Drupal::entityTypeManager()->getStorage('file')->loadByProperties(['filename' => $filename]);
      $file_obj = reset($files);
      (new self)->compress($file_obj);
    }
    $context['sandbox']['current_id'] += $limit;
    if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }


  /**
   * Function for batch process finished.
   *
   * @param int $success
   *   Variable stores the success result.
   */
  public static function imageCompressionBatchProcessFinished($success, $results, $op) {
    if ($success) {
      $message = "Images processed";
    } else {
      $message = "Image Processing Failed";
    }
    \Drupal::messenger()->addMessage(t('Images Processed.'));
  }
}
